@@ -454,16 +454,18 @@
 	     (skk-update-mode id sc)
 	     #f)
 	   #t)
-       (if (shift-key-mask key-state)
+       (if (and (shift-key-mask key-state)
+		(skk-context-head sc))
 	   (begin
 	     (skk-context-set-state! sc 'skk-state-okuri)
 	     (skk-update-mode id sc)
-	     (set! key (to-lower-char key))
+	     (set! key (to-lower-char key))	     
 	     (skk-context-set-okuri-head! sc
 					  (charcode->string key))
 	     #t)
 	   #t)
        (begin
+	 (set! key (to-lower-char key))	     
 	 (set! stat (skk-context-state sc))
 	 (set! res
 	       (rk-push-key!
