@@ -83,6 +83,21 @@ uim_flush_cb(uim_context uc)
 	  uc->mode_list_update_cb(uc->ptr);
 	}
       }
+      break;
+    case PROP_LABEL_UPDATE_CB:
+      {
+	if (uc->prop_label_update_cb) {
+	  uc->prop_label_update_cb(uc->ptr, uc->proplabelstr);
+	}
+      }
+      break;
+    case PROP_LIST_UPDATE_CB:
+      {
+	if (uc->prop_list_update_cb) {
+	  uc->prop_list_update_cb(uc->ptr, uc->propstr);
+	}
+      }
+      break;
     default:;
     }
     if (tmp->str) {
@@ -121,7 +136,12 @@ uim_code_conv(char *str, iconv_t ic)
 void
 uim_schedule_cb(uim_context uc, int type, char *str, int n1, int n2)
 {
-  struct cb *cb = malloc(sizeof(struct cb));
+  struct cb *cb;
+
+  if(!uc)
+    return;
+
+  cb = malloc(sizeof(struct cb));
   cb->type = type;
   cb->str = str;
   cb->n1 = n1;
@@ -202,6 +222,10 @@ im_set_encoding(LISP id, LISP enc)
 {
   char *e = uim_get_c_string(enc);
   uim_context uc = uim_find_context(get_c_long(id));
+
+  if(!uc)
+    return NIL;
+
   if (uc->conv) {
     iconv_close(uc->conv);
   }
@@ -223,6 +247,10 @@ im_clear_mode_list(LISP id)
 {
   int i;
   uim_context uc = uim_find_context(get_c_long(id));
+
+  if(!uc)
+    return NIL;
+
   for (i = 0; i < uc->nr_modes; i++) {
     if (uc->modes[i]) {
       free(uc->modes[i]);
@@ -241,6 +269,10 @@ im_pushback_mode_list(LISP id, LISP str)
 {
   char *s;
   uim_context uc = uim_find_context(get_c_long(id));
+
+  if(!uc)
+    return NIL;
+
   uc->modes = realloc(uc->modes,
 		      sizeof(char *)*(uc->nr_modes+1));
   s = uim_get_c_string(str);
@@ -254,15 +286,59 @@ static LISP
 im_update_mode_list(LISP id)
 {
   uim_context uc = uim_find_context(get_c_long(id));
+
+  if(!uc)
+    return NIL;
+
   uim_schedule_cb(uc, MODE_LIST_UPDATE_CB, NULL, 0, 0);
   return NIL;
 }
 
+static LISP
+im_update_prop_list(LISP id, LISP prop_)
+{
+  uim_context uc = uim_find_context(get_c_long(id));
+  char *prop     = uim_get_c_string(prop_);
+  char *str;
+
+  if(uc)
+    uim_schedule_cb(uc, PROP_LIST_UPDATE_CB, NULL, 0, 0);
+  
+  if(uc->propstr)
+    free(uc->propstr);
+      
+  uc->propstr = uim_code_conv(prop, uc->conv);
+  
+  return NIL;
+}
+
+
+static LISP
+im_update_prop_label(LISP id, LISP prop_)
+{
+  uim_context uc = uim_find_context(get_c_long(id));
+  char *prop     = uim_get_c_string(prop_);
+    
+  if(uc)
+    uim_schedule_cb(uc, PROP_LABEL_UPDATE_CB, NULL, 0, 0);
+  
+  if(uc->proplabelstr)
+    free(uc->proplabelstr);
+  
+  uc->proplabelstr = uim_code_conv(prop, uc->conv);
+  
+  return NIL;
+}
+
 static LISP
 im_update_mode(LISP id, LISP mode_)
 {
   int mode = get_c_long(mode_);
   uim_context uc = uim_find_context(get_c_long(id));
+
+  if(!uc)
+    return NIL;
+
   uc->mode = mode;
   uim_schedule_cb(uc, MODE_UPDATE_CB, NULL, mode, 0);
   return NIL;
@@ -356,23 +432,26 @@ uim_init_im_subrs()
   /**/
   init_subr_1("im-return-str", im_return_str);
   /**/
-  init_subr_2("im-commit", im_commit);
-  init_subr_1("im-commit-raw", im_commit_raw);
+  init_subr_2("im-commit",       im_commit);
+  init_subr_1("im-commit-raw",   im_commit_raw);
   init_subr_2("im-set-encoding", im_set_encoding);
   /**/
   init_subr_3("im-register-im", im_register_im);
   /**/
-  init_subr_1("im-clear-preedit", im_clear_preedit);
+  init_subr_1("im-clear-preedit",    im_clear_preedit);
   init_subr_3("im-pushback-preedit", im_pushback_preedit);
-  init_subr_1("im-update-preedit", im_update_preedit);
+  init_subr_1("im-update-preedit",   im_update_preedit);
   /**/
-  init_subr_1("im-clear-mode-list", im_clear_mode_list);
+  init_subr_1("im-clear-mode-list",    im_clear_mode_list);
   init_subr_2("im-pushback-mode-list", im_pushback_mode_list);
-  init_subr_1("im-update-mode-list", im_update_mode_list);
+  init_subr_1("im-update-mode-list",   im_update_mode_list);
+  /**/
+  init_subr_1("im-update-prop-label", im_update_prop_label);
+  init_subr_1("im-update-prop-list",  im_update_prop_list);
   /**/
   init_subr_2("im-update-mode", im_update_mode);
   /**/
-  init_subr_3("im-begin-candidate", im_begin_candidate);
+  init_subr_3("im-begin-candidate",  im_begin_candidate);
   init_subr_2("im-update-candidate", im_update_candidate);
-  init_subr_1("im-end-candidate", im_end_candidate);
+  init_subr_1("im-end-candidate",    im_end_candidate);
 }
