@@ -24,7 +24,6 @@ static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
 static int uim_initialized;
-int uim_fd = -1;
 
 void
 uim_set_preedit_cb(uim_context uc,
@@ -39,56 +38,23 @@ uim_set_preedit_cb(uim_context uc,
   uc->preedit_update_cb = update_cb;
 }
 
-int
-uim_get_context_id(uim_context is)
+static void
+get_context_id(uim_context is)
 {
   int i;
-
-  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
-    if (context_array[i] == is) {
-      return is->id;
-    }
-    return NULL;
-  }
-}
-
-
-int
-uim_new_context_id(uim_context is)
-{
-  int i;
-
-  is->id = NULL;
-
   for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
     if (!context_array[i]) {
       context_array[i] = is;
-
-      sscanf(uim_helper_send_message(uim_fd, "new_context"),
-	     "%d", &(is->id));      
-      break;	
+      is->id = i;
+      return;
     }
-    
   }
-  if(!is->id)
-    is->id = i + 1;
-  
-  //  printf("new context id:%d\n",(is->id));
-  
-  return is->id; 
 }
 
-
 static void
 put_context_id(uim_context is)
 {
-  int i;
-  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
-    if(context_array[i] && context_array[i]->id == is->id) {
-      context_array[i] = 0;
-      return;
-    }
-  }
+  context_array[is->id] = NULL;
 }
 
 uim_context
@@ -106,7 +72,7 @@ uim_create_context(void *ptr,
   if (!uc) {
     return NULL;
   }
-  uim_new_context_id(uc);
+  get_context_id(uc);
   uc->ptr = ptr;
   uc->commit_cb = commit_cb;
   if (!enc) {
@@ -119,6 +85,9 @@ uim_create_context(void *ptr,
   uc->modes = NULL;
   uc->mode = 0;
   /**/
+  uc->propstr = NULL;
+  uc->proplabelstr = NULL;
+  /**/
   uc->preedit_clear_cb = NULL;
   uc->preedit_pushback_cb = NULL;
   uc->preedit_update_cb = NULL;
@@ -178,13 +147,7 @@ uim_release_context(uim_context uc)
 uim_context
 uim_find_context(int id)
 {
-  int i;
-  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
-    if (context_array[i]->id == id) {
-      return  context_array[i];
-    }
-  }
-  return NULL;
+  return context_array[id];
 }
 
 int
@@ -209,6 +172,24 @@ uim_set_mode_list_update_cb(uim_context uc,
   uc->mode_list_update_cb = update_cb;
 }
 
+
+void
+uim_set_prop_list_update_cb(uim_context uc,
+			    void (*update_cb)(void *ptr))
+{
+  printf("set prop list call %p\n",update_cb);
+  uc->prop_list_update_cb = update_cb;
+}
+
+
+void
+uim_set_prop_label_update_cb(uim_context uc,
+			     void (*update_cb)(void *ptr))
+{
+  uc->prop_label_update_cb = update_cb;
+}
+
+
 int
 uim_get_current_mode(uim_context uc)
 {
@@ -231,6 +212,14 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
   uc->mode_update_cb = update_cb;
 }
 
+void
+uim_prop_list_update(uim_context uc)
+{
+  printf("prop list update\n");
+  uc->prop_list_update_cb(uc, uc->propstr);
+}
+
+
 static int
 can_convert_p(char *from, char *to)
 {
