@@ -105,7 +105,19 @@ int uim_init();
   void
   uim_set_mode_list_update_cb(uim_context uc,
 			      void (*update_cb)(void *ptr));
-  
+  /*property*/
+  void
+  uim_prop_list_update(uim_context uc);
+  void
+  uim_prop_label_update(uim_context uc);
+  void
+  uim_set_prop_list_update_cb(uim_context uc,
+			      void (*update_cb)(void *ptr));
+
+  void
+  uim_set_prop_label_update_cb(uim_context uc,
+			       void (*update_cb)(void *ptr));
+
   /* mode */
   int
   uim_get_current_mode(uim_context uc);
@@ -124,9 +136,6 @@ int uim_init();
   int   uim_get_candidate_index(uim_context uc);
   void  uim_set_candidate_index(uim_context uc, int index);
 
-  char *uim_helper_get_pathname(void);
-  int   uim_helper_init_client_fd(char *path);
-
 #ifdef __cplusplus
 }
 #endif
